@@ -6,6 +6,7 @@
  */
 
 #include <stdbool.h>
+#include <unistd.h>
 
 #include "zpc/aes_key.h"
 #include "zpc/error.h"
@@ -31,6 +32,9 @@ static int aes_key_blob_is_pkey_extractable(struct zpc_aes_key *aes_key,
 								const unsigned char *buf, size_t buflen);
 static int aes_key_add_ep11_header(struct zpc_aes_key *aes_key);
 static int aes_key_blob_has_a_session(struct zpc_aes_key *aes_key);
+int aes_key_pvsec2prot(struct zpc_aes_key *aes_key);
+int aes_key_blob_is_valid_pvsecret_id(struct zpc_aes_key *aes_key,
+								const unsigned char *id);
 
 int
 zpc_aes_key_alloc(struct zpc_aes_key **aes_key)
@@ -70,8 +74,7 @@ zpc_aes_key_alloc(struct zpc_aes_key **aes_key)
 		goto ret;
 	}
 	new_aes_key->refcount = 1;
-	DEBUG("aes key at %p: refcount %llu", new_aes_key,
-	    new_aes_key->refcount);
+	DEBUG("aes key at %p: refcount %llu", new_aes_key, new_aes_key->refcount);
 
 	*aes_key = new_aes_key;
 	rc = 0;
@@ -165,6 +168,13 @@ zpc_aes_key_set_type(struct zpc_aes_key *aes_key, int type)
 	case ZPC_AES_KEY_TYPE_CCA_CIPHER:      /* fall-through */
 	case ZPC_AES_KEY_TYPE_EP11:
 		break;
+	case ZPC_AES_KEY_TYPE_PVSECRET:
+		if (!swcaps.uv_pvsecrets) {
+			rc = ZPC_ERROR_UV_PVSECRETS_NOT_AVAILABLE;
+			DEBUG("return %d (%s)", rc, zpc_error_string(rc));
+			return rc;
+		}
+		break;
 	default:
 		rc = ZPC_ERROR_KEYTYPE;
 		DEBUG("return %d (%s)", rc, zpc_error_string(rc));
@@ -182,8 +192,7 @@ zpc_aes_key_set_type(struct zpc_aes_key *aes_key, int type)
 	if (aes_key->type_set == 1 && aes_key->type != type
 	    && aes_key->mkvp_set == 1) {
 		/* Update mkvp-based apqn choices in case of type change. */
-		DEBUG("aes key at %p: update apqns to match type %d", aes_key,
-		    type);
+		DEBUG("aes key at %p: update apqns to match type %d", aes_key, type);
 		free(aes_key->apqns);
 		aes_key->apqns = NULL;
 		aes_key->napqns = 0;
@@ -309,13 +318,17 @@ zpc_aes_key_set_mkvp(struct zpc_aes_key *aes_key, const char *mkvp)
 	aes_key->napqns = 0;
 	aes_key->apqns_set = 0;
 
+	if (aes_key->type == ZPC_AES_KEY_TYPE_PVSECRET) {
+		rc = 0; /* function has no effect */
+		goto ret;
+	}
+
 	rc = alloc_apqns_from_mkvp(pkeyfd, &(aes_key->apqns), &(aes_key->napqns),
 							mkvpbuf, aes_key->type);
 	if (rc != 0)
 		goto ret;
 
-	DEBUG("aes key at %p: mkvp and %lu apqns set", aes_key,
-	    aes_key->napqns);
+	DEBUG("aes key at %p: mkvp and %lu apqns set", aes_key, aes_key->napqns);
 	memcpy(aes_key->mkvp, mkvpbuf, mkvpbuflen);
 	aes_key->apqns_set = 1;
 	aes_key->mkvp_set = 1;
@@ -362,6 +375,11 @@ zpc_aes_key_set_apqns(struct zpc_aes_key *aes_key, const char *apqns[])
 		goto ret;
 	}
 
+	if (aes_key->type == ZPC_AES_KEY_TYPE_PVSECRET) {
+		rc = 0; /* function has no effect */
+		goto ret;
+	}
+
 	for (napqns = 0; apqns[napqns] != NULL; napqns++);
 
 	if (aes_key->refcount != 1) {
@@ -460,6 +478,10 @@ zpc_aes_key_import_clear(struct zpc_aes_key *aes_key, const unsigned char *key)
 		rc = ZPC_ERROR_KEYTYPENOTSET;
 		goto ret;
 	}
+	if (aes_key->type == ZPC_AES_KEY_TYPE_PVSECRET) {
+		rc = ZPC_ERROR_KEYTYPE;
+		goto ret;
+	}
 	flags = aes_key->flags_set == 1 ? aes_key->flags : 0;
 
 	memset(&aes_key->cur, 0, sizeof(aes_key->cur));
@@ -486,14 +508,14 @@ zpc_aes_key_import_clear(struct zpc_aes_key *aes_key, const unsigned char *key)
 	 * key in both cases.
 	 */
 	rc = ioctl(pkeyfd, PKEY_CLR2SECK2, &clr2seck2);
-	DEBUG("ioctl PKEY_CLR2SECK2 as aes ep11 type 6 key returned %d", rc);
 	if (rc != 0) {
+		DEBUG("ioctl PKEY_CLR2SECK2 for %s key returned %d",
+			aes_key->type == ZPC_AES_KEY_TYPE_EP11 ? "ep11 aes type 6" : "cca", rc);
 		if (aes_key->type == ZPC_AES_KEY_TYPE_EP11) {
-			DEBUG("ioctl PKEY_CLR2SECK2 as aes ep11 type 6 key returned %d", rc);
 			/* Retry with a type 3 key */
 			clr2seck2.type = TOKVER_EP11_AES; /* 0x03 */
 			rc = ioctl(pkeyfd, PKEY_CLR2SECK2, &clr2seck2);
-			DEBUG("ioctl PKEY_CLR2SECK2 as aes ep11 type 3 key returned %d", rc);
+			DEBUG("ioctl PKEY_CLR2SECK2 for aes ep11 type 3 key returned %d", rc);
 			if (rc != 0) {
 				rc = ZPC_ERROR_IOCTLCLR2SECK2;
 				goto ret;
@@ -616,9 +638,6 @@ zpc_aes_key_import(struct zpc_aes_key *aes_key, const unsigned char *buf,
 	if (buf == NULL) {
 		return ZPC_ERROR_ARG2NULL;
 	}
-	if (buflen < MIN_SECURE_KEY_SIZE || buflen > MAX_SECURE_KEY_SIZE) {
-		return ZPC_ERROR_ARG3RANGE;
-	}
 
 	rv = pthread_mutex_lock(&aes_key->lock);
 	assert(rv == 0);
@@ -627,7 +646,17 @@ zpc_aes_key_import(struct zpc_aes_key *aes_key, const unsigned char *buf,
 		rc = ZPC_ERROR_OBJINUSE;
 		goto ret;
 	}
-
+	if (aes_key->type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		if (buflen < MIN_SECURE_KEY_SIZE || buflen > MAX_SECURE_KEY_SIZE) {
+			rc = ZPC_ERROR_ARG3RANGE;
+			goto ret;
+		}
+	} else {
+		if (buflen != UV_SECRET_ID_LEN) {
+			rc = ZPC_ERROR_ARG3RANGE;
+			goto ret;
+		}
+	}
 	if (aes_key->keysize_set != 1) {
 		rc = ZPC_ERROR_KEYSIZENOTSET;
 		goto ret;
@@ -654,14 +683,21 @@ zpc_aes_key_import(struct zpc_aes_key *aes_key, const unsigned char *buf,
 		goto ret;
 	}
 
-	if (!aes_key_blob_has_valid_mkvp(aes_key, buf, buflen)) {
-		rc = ZPC_ERROR_WKVPMISMATCH;
-		goto ret;
-	}
+	if (aes_key->type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		if (!aes_key_blob_has_valid_mkvp(aes_key, buf, buflen)) {
+			rc = ZPC_ERROR_WKVPMISMATCH;
+			goto ret;
+		}
 
-	if (!aes_key_blob_is_pkey_extractable(aes_key, buf, buflen)) {
-		rc = ZPC_ERROR_BLOB_NOT_PKEY_EXTRACTABLE;
-		goto ret;
+		if (!aes_key_blob_is_pkey_extractable(aes_key, buf, buflen)) {
+			rc = ZPC_ERROR_BLOB_NOT_PKEY_EXTRACTABLE;
+			goto ret;
+		}
+	} else {
+		if (aes_key_blob_is_valid_pvsecret_id(aes_key, buf) != 0) {
+			rc = ZPC_ERROR_PVSECRET_ID_NOT_FOUND_IN_UV;
+			goto ret;
+		}
 	}
 
 	memset(aes_key->cur.sec, 0, sizeof(aes_key->cur.sec));
@@ -712,6 +748,11 @@ zpc_aes_key_generate(struct zpc_aes_key *aes_key)
 		goto ret;
 	}
 
+	if (aes_key->type == ZPC_AES_KEY_TYPE_PVSECRET) {
+		rc = ZPC_ERROR_KEYTYPE;
+		goto ret;
+	}
+
 	if (aes_key->keysize_set != 1) {
 		rc = ZPC_ERROR_KEYSIZENOTSET;
 		goto ret;
@@ -742,10 +783,8 @@ zpc_aes_key_generate(struct zpc_aes_key *aes_key)
 			goto ret;
 		}
 
-		DEBUG("aes key at %p: key set to generated protected key",
-		    aes_key);
-		memcpy(&aes_key->prot, &genprotk.protkey,
-		    sizeof(aes_key->prot));
+		DEBUG("aes key at %p: key set to generated protected key", aes_key);
+		memcpy(&aes_key->prot, &genprotk.protkey, sizeof(aes_key->prot));
 		aes_key->rand_protk = 1;
 		aes_key->key_set = 1;
 		rc = 0;
@@ -780,14 +819,14 @@ zpc_aes_key_generate(struct zpc_aes_key *aes_key)
 	 * key in both cases.
 	 */
 	rc = ioctl(pkeyfd, PKEY_GENSECK2, &genseck2);
-	DEBUG("ioctl PKEY_GENSECK2 as aes ep11 type 6 key returned %d", rc);
 	if (rc != 0) {
+		DEBUG("ioctl PKEY_GENSECK2 for %s key returned %d",
+			aes_key->type == ZPC_AES_KEY_TYPE_EP11 ? "ep11 aes type 6" : "cca", rc);
 		if (aes_key->type == ZPC_AES_KEY_TYPE_EP11) {
-			DEBUG("ioctl PKEY_GENSECK2 as aes ep11 type 6 key returned %d", rc);
 			/* Retry to generate a type 3 key */
 			genseck2.type = TOKVER_EP11_AES; /* 0x03 */;
 			rc = ioctl(pkeyfd, PKEY_GENSECK2, &genseck2);
-			DEBUG("ioctl PKEY_GENSECK2 as aes ep11 type 3 key returned %d", rc);
+			DEBUG("ioctl PKEY_GENSECK2 for aes ep11 type 3 key returned %d", rc);
 			if (rc != 0) {
 				rc = ZPC_ERROR_IOCTLGENSECK2;
 				goto ret;
@@ -811,10 +850,12 @@ zpc_aes_key_generate(struct zpc_aes_key *aes_key)
 			goto ret;
 	}
 
-	if (aes_key->type == ZPC_AES_KEY_TYPE_EP11 && aes_key_blob_has_a_session(aes_key))
-		DEBUG("This aes ep11 key has a session id");
-	else
-		DEBUG("This aes ep11 key has no session id");
+	if (aes_key->type == ZPC_AES_KEY_TYPE_EP11) {
+		if (aes_key_blob_has_a_session(aes_key))
+			DEBUG("This aes ep11 key has a session id");
+		else
+			DEBUG("This aes ep11 key has no session id");
+	}
 
 	DEBUG("aes key at %p: key set to generated secure key", aes_key);
 	aes_key->key_set = 1;
@@ -871,6 +912,11 @@ zpc_aes_key_reencipher(struct zpc_aes_key *aes_key, int method)
 		rc = ZPC_ERROR_KEYTYPENOTSET;
 		goto ret;
 	}
+	if (aes_key->type == ZPC_AES_KEY_TYPE_PVSECRET) {
+		/* reencipher not applicable for pvsecrets */
+		rc = ZPC_ERROR_KEYTYPE;
+		goto ret;
+	}
 	if (aes_key->apqns_set == 0 || aes_key->napqns == 0) {
 		rc = ZPC_ERROR_APQNSNOTSET;
 		goto ret;
@@ -1031,6 +1077,98 @@ __aes_key_reset(struct zpc_aes_key *aes_key)
 	aes_key->refcount = 1;
 }
 
+u16 aesprotkeylen_from_pvsectype(u16 pvsectype)
+{
+	switch (pvsectype) {
+	case ZPC_AES_SECRET_AES_128:
+		return 16 + 32;
+	case ZPC_AES_SECRET_AES_192:
+		return 24 + 32;
+	case ZPC_AES_SECRET_AES_256:
+		return 32 + 32;
+	default:
+		break;
+	}
+
+	return 0;
+}
+
+void aes_key_make_uvrsecrettoken(struct zpc_aes_key *aes_key, const unsigned char *id,
+				unsigned char *buf)
+{
+	struct uvrsecrettoken *clrtok = (struct uvrsecrettoken *)buf;
+
+	clrtok->version = TOKVER_UV_SECRET;
+	switch (aes_key->keysize) {
+	case 128:
+		clrtok->secret_type = ZPC_AES_SECRET_AES_128;
+		break;
+	case 192:
+		clrtok->secret_type = ZPC_AES_SECRET_AES_192;
+		break;
+	default:
+		clrtok->secret_type = ZPC_AES_SECRET_AES_256;
+		break;
+	}
+	clrtok->secret_len = aesprotkeylen_from_pvsectype(clrtok->secret_type);
+	memcpy(clrtok->secret_id, id, UV_SECRET_ID_LEN);
+}
+
+/*
+ * Verify that a given pvsecret ID is a valid ID on this system, i.e. an UV
+ * secret exists with this ID and has the expected key length.
+ */
+int aes_key_blob_is_valid_pvsecret_id(struct zpc_aes_key *aes_key, const unsigned char *id)
+{
+	struct pkey_verifykey2 io;
+	unsigned char buf[sizeof(struct uvrsecrettoken)] = { 0, };
+	int rc;
+
+	aes_key_make_uvrsecrettoken(aes_key, id, buf);
+
+	memset(&io, 0, sizeof(io));
+	io.key = buf;
+	io.keylen = sizeof(struct uvrsecrettoken);
+
+	errno = 0;
+	rc = ioctl(pkeyfd, PKEY_VERIFYKEY2, &io);
+	if (rc != 0) {
+		DEBUG("aes key at %p: PKEY_VERIFYKEY2 ioctl failed, errno = %d", aes_key, errno);
+		return ZPC_ERROR_IOCTLVERIFYKEY2;
+	}
+
+	return 0;
+}
+
+/*
+ * (Re)derive protected key from a retrievable secret ID.
+ * Caller must hold aes_key's wr lock.
+ */
+int aes_key_pvsec2prot(struct zpc_aes_key *aes_key)
+{
+	struct pkey_kblob2pkey3 io;
+	unsigned char buf[sizeof(struct uvrsecrettoken)] = { 0, };
+	int rc;
+
+	aes_key_make_uvrsecrettoken(aes_key, aes_key->cur.sec, buf);
+
+	memset(&io, 0, sizeof(io));
+	io.key = buf;
+	io.keylen = sizeof(struct uvrsecrettoken);
+	io.pkeytype = aes_key->type;
+	io.pkeylen = sizeof(aes_key->prot.protkey);
+	io.pkey = (unsigned char *)&aes_key->prot.protkey;
+
+	errno = 0;
+	rc = ioctl(pkeyfd, PKEY_KBLOB2PROTK3, &io);
+	if (rc != 0) {
+		DEBUG("aes key at %p: PKEY_KBLOB2PROTK3 ioctl failed, errno = %d", aes_key, errno);
+		return ZPC_ERROR_IOCTLBLOB2PROTK3;
+	}
+
+	return 0;
+}
+
 /*
  * (Re)derive protected key from a secure key.
  * Caller must hold aes_key's wr lock.
@@ -1039,7 +1177,7 @@ int aes_key_sec2prot_without_header(struct zpc_aes_key *aes_key, enum aes_key_se
 {
 	struct pkey_kblob2pkey2 io;
 	struct aes_key *key = NULL;
-	int rc;
+	int rc, i;
 
 	assert(sec == AES_KEY_SEC_OLD || sec == AES_KEY_SEC_CUR);
 
@@ -1055,7 +1193,13 @@ int aes_key_sec2prot_without_header(struct zpc_aes_key *aes_key, enum aes_key_se
 	io.apqns = aes_key->apqns;
 	io.apqn_entries = aes_key->napqns;
 
-	rc = ioctl(pkeyfd, PKEY_KBLOB2PROTK2, &io);
+	for (i = 0; i < 10; i++) {
+		rc = ioctl(pkeyfd, PKEY_KBLOB2PROTK2, &io);
+		if (rc == 0 || (errno != -EBUSY && errno != -EAGAIN))
+			break;
+		sleep(1);
+	}
+
 	if (rc != 0)
 		return ZPC_ERROR_IOCTLBLOB2PROTK2;
 
@@ -1072,7 +1216,7 @@ int aes_key_sec2prot_with_header(struct zpc_aes_key *aes_key, enum aes_key_sec s
 {
 	struct pkey_kblob2pkey2 io;
 	struct aes_key *key = NULL;
-	int rc;
+	int rc, i;
 	unsigned char temp[sizeof(struct ep11kblob_header)];
 	struct ep11kblob_header *hdr;
 
@@ -1094,7 +1238,12 @@ int aes_key_sec2prot_with_header(struct zpc_aes_key *aes_key, enum aes_key_sec s
 	io.apqns = aes_key->apqns;
 	io.apqn_entries = aes_key->napqns;
 
-	rc = ioctl(pkeyfd, PKEY_KBLOB2PROTK2, &io);
+	for (i = 0; i < 10; i++) {
+		rc = ioctl(pkeyfd, PKEY_KBLOB2PROTK2, &io);
+		if (rc == 0 || (errno != -EBUSY && errno != -EAGAIN))
+			break;
+		sleep(1);
+	}
 	if (rc == 0)
 		goto done;
 
@@ -1124,7 +1273,12 @@ int aes_key_sec2prot_with_header(struct zpc_aes_key *aes_key, enum aes_key_sec s
 	io.apqns = aes_key->apqns;
 	io.apqn_entries = aes_key->napqns;
 
-	rc = ioctl(pkeyfd, PKEY_KBLOB2PROTK2, &io);
+	for (i = 0; i < 10; i++) {
+		rc = ioctl(pkeyfd, PKEY_KBLOB2PROTK2, &io);
+		if (rc == 0 || (errno != -EBUSY && errno != -EAGAIN))
+			break;
+		sleep(1);
+	}
 
 	memcpy(key->sec + 16, temp, 16); // restore session id in any case
 
@@ -1142,23 +1296,29 @@ done:
  */
 int aes_key_sec2prot(struct zpc_aes_key *aes_key, enum aes_key_sec sec)
 {
-	if (aes_key->type == ZPC_AES_KEY_TYPE_EP11) {
-		struct aes_key *key = NULL;
-		size_t keylen;
+	struct aes_key *key = NULL;
+	size_t keylen;
+
+	switch (aes_key->type) {
+	case ZPC_AES_KEY_TYPE_EP11:
 		assert(sec == AES_KEY_SEC_OLD || sec == AES_KEY_SEC_CUR);
 		if (sec == AES_KEY_SEC_CUR) {
 			key = &aes_key->cur;
 			keylen = aes_key->cur.seclen;
-		} else if (sec == AES_KEY_SEC_OLD) {
+		} else {
 			key = &aes_key->old;
 			keylen = aes_key->old.seclen;
 		}
 		assert(key != NULL);
 		if (is_ep11_aes_key_with_header(key->sec, keylen))
 			return aes_key_sec2prot_with_header(aes_key, sec);
+		else
+			return aes_key_sec2prot_without_header(aes_key, sec);
+	case ZPC_AES_KEY_TYPE_PVSECRET:
+		return aes_key_pvsec2prot(aes_key);
+	default:
+		return aes_key_sec2prot_without_header(aes_key, sec);
 	}
-
-	return aes_key_sec2prot_without_header(aes_key, sec);
 }
 
 int aes_key_clr2prot(struct zpc_aes_key *aes_key, const unsigned char *key,
