@@ -80,16 +80,18 @@ TEST(aes_ecb, set_key)
 
 	TESTLIB_AES_ECB_HW_CAPS_CHECK();
 
-	TESTLIB_AES_KERNEL_CAPS_CHECK();
-
 	size = testlib_env_aes_key_size();
 	type = testlib_env_aes_key_type();
 	flags = testlib_env_aes_key_flags();
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_KERNEL_CAPS_CHECK(type);
+
 	TESTLIB_AES_SW_CAPS_CHECK(type);
 
+	TESTLIB_APQN_CAPS_CHECK(apqns, mkvp, type, size, flags);
+
 	rc = zpc_aes_key_alloc(&aes_key);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_ecb_alloc(&aes_ecb);
@@ -111,8 +113,15 @@ TEST(aes_ecb, set_key)
 	}
 	rc = zpc_aes_key_set_flags(aes_key, flags);
 	EXPECT_EQ(rc, 0);
-	rc = zpc_aes_key_import_clear(aes_key, clearkey);
-	EXPECT_EQ(rc, 0);
+
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		rc = zpc_aes_key_import_clear(aes_key, clearkey);
+		EXPECT_EQ(rc, 0);
+	} else {
+		rc = testlib_set_aes_key_from_pvsecret(aes_key, size);
+		if (rc)
+			goto ret;
+	}
 
 	rc = zpc_aes_ecb_set_key(NULL, NULL);
 	EXPECT_EQ(rc, ZPC_ERROR_ARG1NULL);
@@ -122,6 +131,7 @@ TEST(aes_ecb, set_key)
 	rc = zpc_aes_ecb_set_key(aes_ecb, NULL);
 	EXPECT_EQ(rc, 0);
 
+ret:
 	zpc_aes_ecb_free(&aes_ecb);
 	EXPECT_EQ(aes_ecb, nullptr);
 	zpc_aes_key_free(&aes_key);
@@ -141,16 +151,18 @@ TEST(aes_ecb, encrypt)
 
 	TESTLIB_AES_ECB_HW_CAPS_CHECK();
 
-	TESTLIB_AES_KERNEL_CAPS_CHECK();
-
 	size = testlib_env_aes_key_size();
 	type = testlib_env_aes_key_type();
 	flags = testlib_env_aes_key_flags();
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_KERNEL_CAPS_CHECK(type);
+
 	TESTLIB_AES_SW_CAPS_CHECK(type);
 
+	TESTLIB_APQN_CAPS_CHECK(apqns, mkvp, type, size, flags);
+
 	rc = zpc_aes_key_alloc(&aes_key);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_ecb_alloc(&aes_ecb);
@@ -169,8 +181,15 @@ TEST(aes_ecb, encrypt)
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_key_set_flags(aes_key, flags);
 	EXPECT_EQ(rc, 0);
-	rc = zpc_aes_key_generate(aes_key);
-	EXPECT_EQ(rc, 0);
+
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		rc = zpc_aes_key_generate(aes_key);
+		EXPECT_EQ(rc, 0);
+	} else {
+		rc = testlib_set_aes_key_from_pvsecret(aes_key, size);
+		if (rc)
+			goto ret;
+	}
 
 	rc = zpc_aes_ecb_set_key(aes_ecb, aes_key);
 	EXPECT_EQ(rc, 0);
@@ -178,6 +197,7 @@ TEST(aes_ecb, encrypt)
 	rc = zpc_aes_ecb_encrypt(aes_ecb, c, m, 64);
 	EXPECT_EQ(rc, 0);
 
+ret:
 	zpc_aes_ecb_free(&aes_ecb);
 	EXPECT_EQ(aes_ecb, nullptr);
 	zpc_aes_key_free(&aes_key);
@@ -197,16 +217,18 @@ TEST(aes_ecb, decrypt)
 
 	TESTLIB_AES_ECB_HW_CAPS_CHECK();
 
-	TESTLIB_AES_KERNEL_CAPS_CHECK();
-
 	size = testlib_env_aes_key_size();
 	type = testlib_env_aes_key_type();
 	flags = testlib_env_aes_key_flags();
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_KERNEL_CAPS_CHECK(type);
+
 	TESTLIB_AES_SW_CAPS_CHECK(type);
 
+	TESTLIB_APQN_CAPS_CHECK(apqns, mkvp, type, size, flags);
+
 	rc = zpc_aes_key_alloc(&aes_key);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_ecb_alloc(&aes_ecb);
@@ -225,14 +247,23 @@ TEST(aes_ecb, decrypt)
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_key_set_flags(aes_key, flags);
 	EXPECT_EQ(rc, 0);
-	rc = zpc_aes_key_generate(aes_key);
-	EXPECT_EQ(rc, 0);
+
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		rc = zpc_aes_key_generate(aes_key);
+		EXPECT_EQ(rc, 0);
+	} else {
+		rc = testlib_set_aes_key_from_pvsecret(aes_key, size);
+		if (rc)
+			goto ret;
+	}
+
 	rc = zpc_aes_ecb_set_key(aes_ecb, aes_key);
 	EXPECT_EQ(rc, 0);
 
 	rc = zpc_aes_ecb_decrypt(aes_ecb, m, c, 64);
 	EXPECT_EQ(rc, 0);
 
+ret:
 	zpc_aes_ecb_free(&aes_ecb);
 	EXPECT_EQ(aes_ecb, nullptr);
 	zpc_aes_key_free(&aes_key);
@@ -252,16 +283,18 @@ TEST(aes_ecb, pc)
 
 	TESTLIB_AES_ECB_HW_CAPS_CHECK();
 
-	TESTLIB_AES_KERNEL_CAPS_CHECK();
-
 	size = testlib_env_aes_key_size();
 	type = testlib_env_aes_key_type();
 	flags = testlib_env_aes_key_flags();
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_KERNEL_CAPS_CHECK(type);
+
 	TESTLIB_AES_SW_CAPS_CHECK(type);
 
+	TESTLIB_APQN_CAPS_CHECK(apqns, mkvp, type, size, flags);
+
 	memcpy(m_bak, m, 80);
 
 	rc = zpc_aes_key_alloc(&aes_key1);
@@ -286,8 +319,15 @@ TEST(aes_ecb, pc)
 	}
 	rc = zpc_aes_key_set_flags(aes_key1, flags);
 	EXPECT_EQ(rc, 0);
-	rc = zpc_aes_key_import_clear(aes_key1, key);
-	EXPECT_EQ(rc, 0);
+
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		rc = zpc_aes_key_import_clear(aes_key1, key);
+		EXPECT_EQ(rc, 0);
+	} else {
+		rc = testlib_set_aes_key_from_pvsecret(aes_key1, size);
+		if (rc)
+			goto ret;
+	}
 	rc = zpc_aes_ecb_set_key(aes_ecb1, aes_key1);
 	EXPECT_EQ(rc, 0);
 
@@ -304,8 +344,14 @@ TEST(aes_ecb, pc)
 	}
 	rc = zpc_aes_key_set_flags(aes_key2, flags);
 	EXPECT_EQ(rc, 0);
-	rc = zpc_aes_key_import_clear(aes_key2, key);
-	EXPECT_EQ(rc, 0);
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		rc = zpc_aes_key_import_clear(aes_key2, key);
+		EXPECT_EQ(rc, 0);
+	} else {
+		rc = testlib_set_aes_key_from_pvsecret(aes_key2, size);
+		if (rc)
+			goto ret;
+	}
 	rc = zpc_aes_ecb_set_key(aes_ecb2, aes_key2);
 	EXPECT_EQ(rc, 0);
 
@@ -329,8 +375,16 @@ TEST(aes_ecb, pc)
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_key_set_size(aes_key1, size);
 	EXPECT_EQ(rc, 0);
-	rc = zpc_aes_key_generate(aes_key1);
-	EXPECT_EQ(rc, 0);
+
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		rc = zpc_aes_key_generate(aes_key1);
+		EXPECT_EQ(rc, 0);
+	} else {
+		rc = testlib_set_aes_key_from_pvsecret(aes_key1, size);
+		if (rc)
+			goto ret;
+	}
+
 	rc = zpc_aes_ecb_set_key(aes_ecb1, aes_key1);
 	EXPECT_EQ(rc, 0);
 
@@ -348,6 +402,7 @@ TEST(aes_ecb, pc)
 	EXPECT_EQ(rc, 0);
 	EXPECT_TRUE(memcmp(m, m_bak, 80) == 0);
 
+ret:
 	zpc_aes_ecb_free(&aes_ecb2);
 	zpc_aes_ecb_free(&aes_ecb1);
 	zpc_aes_key_free(&aes_key2);
@@ -362,8 +417,6 @@ TEST(aes_ecb, stream_inplace_kat)
 
 	TESTLIB_AES_ECB_HW_CAPS_CHECK();
 
-	TESTLIB_AES_KERNEL_CAPS_CHECK();
-
 	size_t keylen, msglen, ctlen;
 	unsigned char buf[4096];
 	const char *mkvp, *apqns[257];
@@ -381,8 +434,15 @@ TEST(aes_ecb, stream_inplace_kat)
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_KERNEL_CAPS_CHECK(type);
+
 	TESTLIB_AES_SW_CAPS_CHECK(type);
 
+	TESTLIB_APQN_CAPS_CHECK(apqns, mkvp, type, 256, flags);
+
+	if (type == ZPC_AES_KEY_TYPE_PVSECRET)
+		GTEST_SKIP_("Skipping stream_inplace_kat test. KATs cannot be performed with UV secrets.");
+
 	u8 *key = testlib_hexstr2buf(keystr, &keylen);
 	ASSERT_NE(key, nullptr);
 	u8 *msg = testlib_hexstr2buf(msgstr, &msglen);
@@ -459,11 +519,20 @@ TEST(aes_ecb, stream_inplace_kat)
 
 TEST(aes_ecb, nist_kat)
 {
+	int type;
+
+	type = testlib_env_aes_key_type();
+
 	TESTLIB_ENV_AES_KEY_CHECK();
 
 	TESTLIB_AES_ECB_HW_CAPS_CHECK();
 
-	TESTLIB_AES_KERNEL_CAPS_CHECK();
+	TESTLIB_AES_KERNEL_CAPS_CHECK(type);
+
+	TESTLIB_AES_SW_CAPS_CHECK(type);
+
+	if (type == ZPC_AES_KEY_TYPE_PVSECRET)
+		GTEST_SKIP_("Skipping nist_kat test. KATs cannot be performed with UV secrets.");
 
 	__run_json("nist_aes_ecb.json");
 }
@@ -490,6 +559,8 @@ static void __run_json(const char *json)
 
 	TESTLIB_AES_SW_CAPS_CHECK(type);
 
+	TESTLIB_APQN_CAPS_CHECK(apqns, mkvp, type, 256, flags);
+
 	rc = zpc_aes_key_alloc(&aes_key);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_ecb_alloc(&aes_ecb);
@@ -595,15 +666,16 @@ TEST(aes_ecb, rederive_protected_key1)
 	struct zpc_aes_key *aes_key1;
 	struct zpc_aes_ecb *aes_ecb1, *aes_ecb2, *aes_ecb3;
 	u8 m[96], c[96];
-	int rc, size;
+	int rc, size, type;
 
 	TESTLIB_ENV_AES_KEY_CHECK();
 
 	TESTLIB_AES_ECB_HW_CAPS_CHECK();
 
-	TESTLIB_AES_KERNEL_CAPS_CHECK();
-
 	size = testlib_env_aes_key_size();
+	type = testlib_env_aes_key_type();
+
+	TESTLIB_AES_KERNEL_CAPS_CHECK(type);
 
 	rc = zpc_aes_key_alloc(&aes_key1);
 	EXPECT_EQ(rc, 0);
@@ -627,6 +699,12 @@ TEST(aes_ecb, rederive_protected_key1)
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_key_set_size(aes_key1, size);
 	EXPECT_EQ(rc, 0);
+	/*
+	 * This key obj has no type set. Therefore the generate will work also
+	 * for tests with ZPC_TEST_AES_KEY_TYPE = ZPC_AES_KEY_TYPE_PVSECRET.
+	 * The generated protected key has no dependency on any secure key or
+	 * pvsecret.
+	 */
 	rc = zpc_aes_key_generate(aes_key1);
 	EXPECT_EQ(rc, 0);
 
@@ -661,8 +739,6 @@ TEST(aes_ecb, rederive_protected_key2)
 
 	TESTLIB_AES_ECB_HW_CAPS_CHECK();
 
-	TESTLIB_AES_KERNEL_CAPS_CHECK();
-
 	size_t keylen, msglen, ctlen;
 	unsigned char buf[4096];
 	const char *mkvp, *apqns[257];
@@ -680,8 +756,12 @@ TEST(aes_ecb, rederive_protected_key2)
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_KERNEL_CAPS_CHECK(type);
+
 	TESTLIB_AES_SW_CAPS_CHECK(type);
 
+	TESTLIB_APQN_CAPS_CHECK(apqns, mkvp, type, 256, flags);
+
 	u8 *key = testlib_hexstr2buf(keystr, &keylen);
 	ASSERT_NE(key, nullptr);
 	u8 *msg = testlib_hexstr2buf(msgstr, &msglen);
@@ -709,8 +789,14 @@ TEST(aes_ecb, rederive_protected_key2)
 	rc = zpc_aes_key_set_size(aes_key, keylen * 8);
 	EXPECT_EQ(rc, 0);
 
-	rc = zpc_aes_key_import_clear(aes_key, key);
-	EXPECT_EQ(rc, 0);
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		rc = zpc_aes_key_import_clear(aes_key, key);
+		EXPECT_EQ(rc, 0);
+	} else {
+		rc = testlib_set_aes_key_from_pvsecret(aes_key, keylen * 8);
+		if (rc)
+			goto ret;
+	}
 
 	/* Encrypt */
 	memcpy(buf, msg, msglen);
@@ -724,7 +810,9 @@ TEST(aes_ecb, rederive_protected_key2)
 	rc = zpc_aes_ecb_encrypt(aes_ecb, buf + 16, buf + 16, msglen - 16);
 	EXPECT_EQ(rc, 0);
 
-	EXPECT_TRUE(memcmp(buf, ct, ctlen) == 0);
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		EXPECT_TRUE(memcmp(buf, ct, ctlen) == 0);
+	}
 
 	/* Encrypt */
 	memcpy(buf, msg, msglen);
@@ -739,7 +827,9 @@ TEST(aes_ecb, rederive_protected_key2)
 	rc = zpc_aes_ecb_encrypt(aes_ecb, buf + 16, buf + 16, msglen - 16);
 	EXPECT_EQ(rc, 0);
 
-	EXPECT_TRUE(memcmp(buf, ct, ctlen) == 0);
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		EXPECT_TRUE(memcmp(buf, ct, ctlen) == 0);
+	}
 
 	/* Decrypt */
 	memcpy(buf, ct, ctlen);
@@ -750,7 +840,9 @@ TEST(aes_ecb, rederive_protected_key2)
 	rc = zpc_aes_ecb_decrypt(aes_ecb, buf + 16, buf + 16, msglen - 16);
 	EXPECT_EQ(rc, 0);
 
-	EXPECT_TRUE(memcmp(buf, msg, msglen) == 0);
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		EXPECT_TRUE(memcmp(buf, msg, msglen) == 0);
+	}
 
 	/* Decrypt */
 	memcpy(buf, ct, ctlen);
@@ -761,8 +853,11 @@ TEST(aes_ecb, rederive_protected_key2)
 	rc = zpc_aes_ecb_decrypt(aes_ecb, buf + 16, buf + 16, msglen - 16);
 	EXPECT_EQ(rc, 0);
 
-	EXPECT_TRUE(memcmp(buf, msg, msglen) == 0);
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		EXPECT_TRUE(memcmp(buf, msg, msglen) == 0);
+	}
 
+ret:
 	zpc_aes_ecb_free(&aes_ecb);
 	EXPECT_EQ(aes_ecb, nullptr);
 	zpc_aes_key_free(&aes_key);
@@ -779,8 +874,6 @@ TEST(aes_ecb, reencipher)
 
 	TESTLIB_AES_ECB_HW_CAPS_CHECK();
 
-	TESTLIB_AES_KERNEL_CAPS_CHECK();
-
 	size_t keylen, msglen, ctlen;
 	unsigned char buf[4096];
 	const char *mkvp, *apqns[257];
@@ -798,10 +891,17 @@ TEST(aes_ecb, reencipher)
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_KERNEL_CAPS_CHECK(type);
+
 	TESTLIB_AES_SW_CAPS_CHECK(type);
 
+	TESTLIB_APQN_CAPS_CHECK(apqns, mkvp, type, 256, flags);
+
 	TESTLIB_AES_NEW_MK_CHECK(type, mkvp, apqns);
 
+	if (type == ZPC_EC_KEY_TYPE_PVSECRET)
+		GTEST_SKIP_("Skipping reencipher test. Not applicable for UV secrets.");
+
 	u8 *key = testlib_hexstr2buf(keystr, &keylen);
 	ASSERT_NE(key, nullptr);
 	u8 *msg = testlib_hexstr2buf(msgstr, &msglen);
@@ -934,10 +1034,15 @@ __task(struct zpc_aes_key *aes_key)
 		rc = zpc_aes_ecb_encrypt(aes_ecb, buf + 16, buf + 16, msglen - 16);
 		EXPECT_EQ(rc, 0);
 
-		EXPECT_TRUE(memcmp(buf, ct, ctlen) == 0);
+		if (aes_key->type != ZPC_AES_KEY_TYPE_PVSECRET) {
+			/* The pvsecret key is not the KAT key */
+			EXPECT_TRUE(memcmp(buf, ct, ctlen) == 0);
+		}
 
 		/* Decrypt */
-		memcpy(buf, ct, ctlen);
+		if (aes_key->type != ZPC_AES_KEY_TYPE_PVSECRET) {
+			memcpy(buf, ct, ctlen);
+		}
 
 		memset(aes_ecb->param.protkey, 0, sizeof(aes_ecb->param.protkey));    /* force WKaVP mismatch */
 		rc = zpc_aes_ecb_decrypt(aes_ecb, buf, buf, 16);
@@ -961,8 +1066,6 @@ TEST(aes_ecb, threads)
 
 	TESTLIB_AES_ECB_HW_CAPS_CHECK();
 
-	TESTLIB_AES_KERNEL_CAPS_CHECK();
-
 	size_t keylen;
 	const char *mkvp, *apqns[257];
 	struct zpc_aes_key *aes_key;
@@ -977,8 +1080,12 @@ TEST(aes_ecb, threads)
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_KERNEL_CAPS_CHECK(type);
+
 	TESTLIB_AES_SW_CAPS_CHECK(type);
 
+	TESTLIB_APQN_CAPS_CHECK(apqns, mkvp, type, 256, flags);
+
 	TESTLIB_AES_NEW_MK_CHECK(type, mkvp, apqns);
 
 	u8 *key = testlib_hexstr2buf(keystr, &keylen);
@@ -1001,18 +1108,26 @@ TEST(aes_ecb, threads)
 	rc = zpc_aes_key_set_size(aes_key, keylen * 8);
 	EXPECT_EQ(rc, 0);
 
-	rc = zpc_aes_key_import_clear(aes_key, key);
-	EXPECT_EQ(rc, 0);
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		rc = zpc_aes_key_import_clear(aes_key, key);
+		EXPECT_EQ(rc, 0);
+	} else {
+		rc = testlib_set_aes_key_from_pvsecret(aes_key, keylen * 8);
+		if (rc)
+			goto ret;
+	}
 
 	for (i = 0; i < 500; i++) {
 		t[i] = new std::thread(__task, aes_key);
 	}
 
 	/* Do something with key object while threads are working with it. */
-	rc = zpc_aes_key_reencipher(aes_key, ZPC_AES_KEY_REENCIPHER_CURRENT_TO_NEW);
-	EXPECT_EQ(rc, 0);
-	memset(&aes_key->cur, 0, sizeof(aes_key->cur));     /* destroy current secure key */
- 
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		rc = zpc_aes_key_reencipher(aes_key, ZPC_AES_KEY_REENCIPHER_CURRENT_TO_NEW);
+		EXPECT_EQ(rc, 0);
+		memset(&aes_key->cur, 0, sizeof(aes_key->cur));     /* destroy current secure key */
+	}
+
 	for (i = 0; i < 500; i++) {
 		memset(&aes_key->prot, 0, sizeof(aes_key->prot));    /* destroy cached protected key */
 		usleep(1);
@@ -1023,6 +1138,7 @@ TEST(aes_ecb, threads)
 		delete t[i];
 	}
 
+ret:
 	zpc_aes_key_free(&aes_key);
 	EXPECT_EQ(aes_key, nullptr);
 
