@@ -80,16 +80,18 @@ TEST(aes_gcm, set_key)
 
 	TESTLIB_AES_GCM_HW_CAPS_CHECK();
 
-	TESTLIB_AES_KERNEL_CAPS_CHECK();
-
 	size = testlib_env_aes_key_size();
 	type = testlib_env_aes_key_type();
 	flags = testlib_env_aes_key_flags();
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_KERNEL_CAPS_CHECK(type);
+
 	TESTLIB_AES_SW_CAPS_CHECK(type);
 
+	TESTLIB_APQN_CAPS_CHECK(apqns, mkvp, type, size, flags);
+
 	rc = zpc_aes_key_alloc(&aes_key);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_gcm_alloc(&aes_gcm);
@@ -111,8 +113,15 @@ TEST(aes_gcm, set_key)
 	}
 	rc = zpc_aes_key_set_flags(aes_key, flags);
 	EXPECT_EQ(rc, 0);
-	rc = zpc_aes_key_import_clear(aes_key, clearkey);
-	EXPECT_EQ(rc, 0);
+
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		rc = zpc_aes_key_import_clear(aes_key, clearkey);
+		EXPECT_EQ(rc, 0);
+	} else {
+		rc = testlib_set_aes_key_from_pvsecret(aes_key, size);
+		if (rc)
+			goto ret;
+	}
 
 	rc = zpc_aes_gcm_set_key(NULL, NULL);
 	EXPECT_EQ(rc, ZPC_ERROR_ARG1NULL);
@@ -129,6 +138,7 @@ TEST(aes_gcm, set_key)
 	rc = zpc_aes_gcm_set_iv(aes_gcm, iv, 12);
 	EXPECT_EQ(rc, 0);
 
+ret:
 	zpc_aes_gcm_free(&aes_gcm);
 	EXPECT_EQ(aes_gcm, nullptr);
 	zpc_aes_key_free(&aes_key);
@@ -148,16 +158,18 @@ TEST(aes_gcm, set_iv)
 
 	TESTLIB_AES_GCM_HW_CAPS_CHECK();
 
-	TESTLIB_AES_KERNEL_CAPS_CHECK();
-
 	size = testlib_env_aes_key_size();
 	type = testlib_env_aes_key_type();
 	flags = testlib_env_aes_key_flags();
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_KERNEL_CAPS_CHECK(type);
+
 	TESTLIB_AES_SW_CAPS_CHECK(type);
 
+	TESTLIB_APQN_CAPS_CHECK(apqns, mkvp, type, size, flags);
+
 	rc = zpc_aes_key_alloc(&aes_key);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_gcm_alloc(&aes_gcm);
@@ -186,8 +198,15 @@ TEST(aes_gcm, set_iv)
 		rc = zpc_aes_key_set_apqns(aes_key, apqns);
 		EXPECT_EQ(rc, 0);
 	}
-	rc = zpc_aes_key_generate(aes_key);
-	EXPECT_EQ(rc, 0);
+
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		rc = zpc_aes_key_generate(aes_key);
+		EXPECT_EQ(rc, 0);
+	} else {
+		rc = testlib_set_aes_key_from_pvsecret(aes_key, size);
+		if (rc)
+			goto ret;
+	}
 
 	rc = zpc_aes_gcm_set_iv(aes_gcm, iv, 12);
 	EXPECT_EQ(rc, ZPC_ERROR_KEYNOTSET);
@@ -204,6 +223,7 @@ TEST(aes_gcm, set_iv)
 	rc = zpc_aes_gcm_set_iv(aes_gcm, iv, 16);
 	EXPECT_EQ(rc, 0);
 
+ret:
 	zpc_aes_gcm_free(&aes_gcm);
 	EXPECT_EQ(aes_gcm, nullptr);
 	zpc_aes_key_free(&aes_key);
@@ -224,16 +244,18 @@ TEST(aes_gcm, create_iv)
 
 	TESTLIB_AES_GCM_HW_CAPS_CHECK();
 
-	TESTLIB_AES_KERNEL_CAPS_CHECK();
-
 	size = testlib_env_aes_key_size();
 	type = testlib_env_aes_key_type();
 	flags = testlib_env_aes_key_flags();
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_KERNEL_CAPS_CHECK(type);
+
 	TESTLIB_AES_SW_CAPS_CHECK(type);
 
+	TESTLIB_APQN_CAPS_CHECK(apqns, mkvp, type, size, flags);
+
 	rc = zpc_aes_key_alloc(&aes_key);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_gcm_alloc(&aes_gcm1);
@@ -260,8 +282,15 @@ TEST(aes_gcm, create_iv)
 		EXPECT_EQ(rc, 0);
 	}
 
-	rc = zpc_aes_key_generate(aes_key);
-	EXPECT_EQ(rc, 0);
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		rc = zpc_aes_key_generate(aes_key);
+		EXPECT_EQ(rc, 0);
+	} else {
+		rc = testlib_set_aes_key_from_pvsecret(aes_key, size);
+		if (rc)
+			goto ret;
+	}
+
 	rc = zpc_aes_gcm_set_key(aes_gcm1, aes_key);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_gcm_set_key(aes_gcm2, aes_key);
@@ -305,6 +334,7 @@ TEST(aes_gcm, create_iv)
 	EXPECT_EQ(rc, 0);
 	EXPECT_TRUE(memcmp(pt, m, sizeof(m)) == 0);
 
+ret:
 	zpc_aes_gcm_free(&aes_gcm1);
 	EXPECT_EQ(aes_gcm1, nullptr);
 	zpc_aes_gcm_free(&aes_gcm2);
@@ -318,7 +348,7 @@ TEST(aes_gcm, encrypt)
 	struct zpc_aes_key *aes_key;
 	struct zpc_aes_gcm *aes_gcm;
 	const char *mkvp, *apqns[257];
-	u8 iv[12], aad[99], m[99], tag[12], c[99];
+	u8 iv[12], aad[99], m[99], tag[12] = { 0, }, c[99];
 	unsigned int flags;
 	int rc, size, type;
 
@@ -326,16 +356,18 @@ TEST(aes_gcm, encrypt)
 
 	TESTLIB_AES_GCM_HW_CAPS_CHECK();
 
-	TESTLIB_AES_KERNEL_CAPS_CHECK();
-
 	size = testlib_env_aes_key_size();
 	type = testlib_env_aes_key_type();
 	flags = testlib_env_aes_key_flags();
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_KERNEL_CAPS_CHECK(type);
+
 	TESTLIB_AES_SW_CAPS_CHECK(type);
 
+	TESTLIB_APQN_CAPS_CHECK(apqns, mkvp, type, size, flags);
+
 	rc = zpc_aes_key_alloc(&aes_key);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_gcm_alloc(&aes_gcm);
@@ -354,8 +386,15 @@ TEST(aes_gcm, encrypt)
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_key_set_flags(aes_key, flags);
 	EXPECT_EQ(rc, 0);
-	rc = zpc_aes_key_generate(aes_key);
-	EXPECT_EQ(rc, 0);
+
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		rc = zpc_aes_key_generate(aes_key);
+		EXPECT_EQ(rc, 0);
+	} else {
+		rc = testlib_set_aes_key_from_pvsecret(aes_key, size);
+		if (rc)
+			goto ret;
+	}
 
 	rc = zpc_aes_gcm_set_key(aes_gcm, aes_key);
 	EXPECT_EQ(rc, 0);
@@ -365,6 +404,7 @@ TEST(aes_gcm, encrypt)
 	rc = zpc_aes_gcm_encrypt(aes_gcm, c, tag, 12, aad, 99, m, 99);
 	EXPECT_EQ(rc, 0);
 
+ret:
 	zpc_aes_gcm_free(&aes_gcm);
 	EXPECT_EQ(aes_gcm, nullptr);
 	zpc_aes_key_free(&aes_key);
@@ -376,7 +416,7 @@ TEST(aes_gcm, decrypt)
 	struct zpc_aes_key *aes_key;
 	struct zpc_aes_gcm *aes_gcm;
 	const char *mkvp, *apqns[257];
-	u8 iv[12], aad[99], m[99], tag[12], c[99];
+	u8 iv[12], aad[99], m[99], tag[12] = { 0, }, c[99];
 	unsigned int flags;
 	int rc, size, type;
 
@@ -384,16 +424,18 @@ TEST(aes_gcm, decrypt)
 
 	TESTLIB_AES_GCM_HW_CAPS_CHECK();
 
-	TESTLIB_AES_KERNEL_CAPS_CHECK();
-
 	size = testlib_env_aes_key_size();
 	type = testlib_env_aes_key_type();
 	flags = testlib_env_aes_key_flags();
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_KERNEL_CAPS_CHECK(type);
+
 	TESTLIB_AES_SW_CAPS_CHECK(type);
 
+	TESTLIB_APQN_CAPS_CHECK(apqns, mkvp, type, size, flags);
+
 	rc = zpc_aes_key_alloc(&aes_key);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_gcm_alloc(&aes_gcm);
@@ -412,8 +454,16 @@ TEST(aes_gcm, decrypt)
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_key_set_flags(aes_key, flags);
 	EXPECT_EQ(rc, 0);
-	rc = zpc_aes_key_generate(aes_key);
-	EXPECT_EQ(rc, 0);
+
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		rc = zpc_aes_key_generate(aes_key);
+		EXPECT_EQ(rc, 0);
+	} else {
+		rc = testlib_set_aes_key_from_pvsecret(aes_key, size);
+		if (rc)
+			goto ret;
+	}
+
 	rc = zpc_aes_gcm_set_key(aes_gcm, aes_key);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_gcm_set_iv(aes_gcm, iv, 12);
@@ -422,6 +472,7 @@ TEST(aes_gcm, decrypt)
 	rc = zpc_aes_gcm_decrypt(aes_gcm, m, tag, 12, aad, 99, c, 99);
 	EXPECT_EQ(rc, ZPC_ERROR_TAGMISMATCH);
 
+ret:
 	zpc_aes_gcm_free(&aes_gcm);
 	EXPECT_EQ(aes_gcm, nullptr);
 	zpc_aes_key_free(&aes_key);
@@ -433,7 +484,7 @@ TEST(aes_gcm, pc)
 	struct zpc_aes_key *aes_key1, *aes_key2;
 	struct zpc_aes_gcm *aes_gcm1, *aes_gcm2;
 	const char *mkvp, *apqns[257];
-	u8 iv[12], aad[99], m[99], tag[12], c[99], key[32], m_bak[99];
+	u8 iv[12], aad[99], m[99], tag[12] = { 0, }, c[99], key[32], m_bak[99];
 	unsigned int flags;
 	int rc, size, type;
 
@@ -441,16 +492,18 @@ TEST(aes_gcm, pc)
 
 	TESTLIB_AES_GCM_HW_CAPS_CHECK();
 
-	TESTLIB_AES_KERNEL_CAPS_CHECK();
-
 	size = testlib_env_aes_key_size();
 	type = testlib_env_aes_key_type();
 	flags = testlib_env_aes_key_flags();
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_KERNEL_CAPS_CHECK(type);
+
 	TESTLIB_AES_SW_CAPS_CHECK(type);
 
+	TESTLIB_APQN_CAPS_CHECK(apqns, mkvp, type, size, flags);
+
 	memcpy(m_bak, m, 99);
 
 	rc = zpc_aes_key_alloc(&aes_key1);
@@ -475,8 +528,16 @@ TEST(aes_gcm, pc)
 	}
 	rc = zpc_aes_key_set_flags(aes_key1, flags);
 	EXPECT_EQ(rc, 0);
-	rc = zpc_aes_key_import_clear(aes_key1, key);
-	EXPECT_EQ(rc, 0);
+
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		rc = zpc_aes_key_import_clear(aes_key1, key);
+		EXPECT_EQ(rc, 0);
+	} else {
+		rc = testlib_set_aes_key_from_pvsecret(aes_key1, size);
+		if (rc)
+			goto ret;
+	}
+
 	rc = zpc_aes_gcm_set_key(aes_gcm1, aes_key1);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_gcm_set_iv(aes_gcm1, iv, 12);
@@ -495,8 +556,16 @@ TEST(aes_gcm, pc)
 	}
 	rc = zpc_aes_key_set_flags(aes_key2, flags);
 	EXPECT_EQ(rc, 0);
-	rc = zpc_aes_key_import_clear(aes_key2, key);
-	EXPECT_EQ(rc, 0);
+
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		rc = zpc_aes_key_import_clear(aes_key2, key);
+		EXPECT_EQ(rc, 0);
+	} else {
+		rc = testlib_set_aes_key_from_pvsecret(aes_key2, size);
+		if (rc)
+			goto ret;
+	}
+
 	rc = zpc_aes_gcm_set_key(aes_gcm2, aes_key2);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_gcm_set_iv(aes_gcm2, iv, 12);
@@ -535,8 +604,15 @@ TEST(aes_gcm, pc)
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_key_set_size(aes_key1, size);
 	EXPECT_EQ(rc, 0);
-	rc = zpc_aes_key_generate(aes_key1);
-	EXPECT_EQ(rc, 0);
+
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		rc = zpc_aes_key_generate(aes_key1);
+		EXPECT_EQ(rc, 0);
+	} else {
+		rc = testlib_set_aes_key_from_pvsecret(aes_key1, size);
+		if (rc)
+			goto ret;
+	}
 
 	rc = zpc_aes_gcm_set_key(aes_gcm1, aes_key1);
 	EXPECT_EQ(rc, 0);
@@ -570,6 +646,7 @@ TEST(aes_gcm, pc)
 	rc = zpc_aes_gcm_decrypt(aes_gcm2, m, tag, 12, aad, 99, c, 99);
 	EXPECT_EQ(rc, ZPC_ERROR_TAGMISMATCH);
 
+ret:
 	zpc_aes_gcm_free(&aes_gcm2);
 	zpc_aes_gcm_free(&aes_gcm1);
 	zpc_aes_key_free(&aes_key2);
@@ -584,8 +661,6 @@ TEST(aes_gcm, stream_inplace_kat1)
 
 	TESTLIB_AES_GCM_HW_CAPS_CHECK();
 
-	TESTLIB_AES_KERNEL_CAPS_CHECK();
-
 	size_t keylen, ivlen, msglen, ctlen, taglen, aadlen;
 	unsigned char buf[4096], mac[16];
 	const char *mkvp, *apqns[257];
@@ -606,8 +681,15 @@ TEST(aes_gcm, stream_inplace_kat1)
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_KERNEL_CAPS_CHECK(type);
+
 	TESTLIB_AES_SW_CAPS_CHECK(type);
 
+	TESTLIB_APQN_CAPS_CHECK(apqns, mkvp, type, 128, flags);
+
+	if (type == ZPC_AES_KEY_TYPE_PVSECRET)
+		GTEST_SKIP_("Skipping stream_inplace_kat1 test. KATs cannot be performed with UV secrets.");
+
 	u8 *key = testlib_hexstr2buf(keystr, &keylen);
 	ASSERT_NE(key, nullptr);
 	u8 *iv = testlib_hexstr2buf(ivstr, &ivlen);
@@ -741,8 +823,6 @@ TEST(aes_gcm, stream_inplace_kat2)
 
 	TESTLIB_AES_GCM_HW_CAPS_CHECK();
 
-	TESTLIB_AES_KERNEL_CAPS_CHECK();
-
 	size_t keylen, ivlen, msglen, ctlen, taglen, aadlen;
 	unsigned char buf[4096], mac[16];
 	const char *mkvp, *apqns[257];
@@ -763,8 +843,15 @@ TEST(aes_gcm, stream_inplace_kat2)
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_KERNEL_CAPS_CHECK(type);
+
 	TESTLIB_AES_SW_CAPS_CHECK(type);
 
+	TESTLIB_APQN_CAPS_CHECK(apqns, mkvp, type, 128, flags);
+
+	if (type == ZPC_AES_KEY_TYPE_PVSECRET)
+		GTEST_SKIP_("Skipping stream_inplace_kat2 test. KATs cannot be performed with UV secrets.");
+
 	u8 *key = testlib_hexstr2buf(keystr, &keylen);
 	ASSERT_NE(key, nullptr);
 	u8 *iv = testlib_hexstr2buf(ivstr, &ivlen);
@@ -849,22 +936,40 @@ TEST(aes_gcm, stream_inplace_kat2)
 
 TEST(aes_gcm, wycheproof_kat)
 {
+	int type;
+
+	type = testlib_env_aes_key_type();
+
 	TESTLIB_ENV_AES_KEY_CHECK();
 
 	TESTLIB_AES_GCM_HW_CAPS_CHECK();
 
-	TESTLIB_AES_KERNEL_CAPS_CHECK();
+	TESTLIB_AES_KERNEL_CAPS_CHECK(type);
+
+	TESTLIB_AES_SW_CAPS_CHECK(type);
+
+	if (type == ZPC_AES_KEY_TYPE_PVSECRET)
+		GTEST_SKIP_("Skipping wycheproof_kat test. KATs cannot be performed with UV secrets.");
 
 	__run_json("wycheproof/src/wycheproof/testvectors/aes_gcm_test.json");
 }
 
 TEST(aes_gcm, nist_kat)
 {
+	int type;
+
+	type = testlib_env_aes_key_type();
+
 	TESTLIB_ENV_AES_KEY_CHECK();
 
 	TESTLIB_AES_GCM_HW_CAPS_CHECK();
 
-	TESTLIB_AES_KERNEL_CAPS_CHECK();
+	TESTLIB_AES_KERNEL_CAPS_CHECK(type);
+
+	TESTLIB_AES_SW_CAPS_CHECK(type);
+
+	if (type == ZPC_AES_KEY_TYPE_PVSECRET)
+		GTEST_SKIP_("Skipping nist_kat test. KATs cannot be performed with UV secrets.");
 
 	__run_json("nist_aes_gcm.json");
 }
@@ -893,6 +998,8 @@ static void __run_json(const char *json)
 
 	TESTLIB_AES_SW_CAPS_CHECK(type);
 
+	TESTLIB_APQN_CAPS_CHECK(apqns, mkvp, type, 256, flags);
+
 	rc = zpc_aes_key_alloc(&aes_key);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_gcm_alloc(&aes_gcm);
@@ -1063,15 +1170,16 @@ TEST(aes_gcm, rederive_protected_key1)
 	struct zpc_aes_key *aes_key1;
 	struct zpc_aes_gcm *aes_gcm1, *aes_gcm2, *aes_gcm3;
 	u8 iv[16], aad[99], m[99], tag[16], c[99]; /* use ivlen != 12 bytes such that it must be processed by kma */
-	int rc, size;
+	int rc, size, type;
 
 	TESTLIB_ENV_AES_KEY_CHECK();
 
 	TESTLIB_AES_GCM_HW_CAPS_CHECK();
 
-	TESTLIB_AES_KERNEL_CAPS_CHECK();
-
 	size = testlib_env_aes_key_size();
+	type = testlib_env_aes_key_type();
+
+	TESTLIB_AES_KERNEL_CAPS_CHECK(type);
 
 	rc = zpc_aes_key_alloc(&aes_key1);
 	EXPECT_EQ(rc, 0);
@@ -1095,6 +1203,12 @@ TEST(aes_gcm, rederive_protected_key1)
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_key_set_size(aes_key1, size);
 	EXPECT_EQ(rc, 0);
+	/*
+	 * This key obj has no type set. Therefore the generate will work also
+	 * for tests with ZPC_TEST_AES_KEY_TYPE = ZPC_AES_KEY_TYPE_PVSECRET.
+	 * The generated protected key has no dependency on any secure key or
+	 * pvsecret.
+	 */
 	rc = zpc_aes_key_generate(aes_key1);
 	EXPECT_EQ(rc, 0);
 
@@ -1144,10 +1258,8 @@ TEST(aes_gcm, rederive_protected_key2)
 
 	TESTLIB_AES_GCM_HW_CAPS_CHECK();
 
-	TESTLIB_AES_KERNEL_CAPS_CHECK();
-
 	size_t keylen, ivlen, msglen, ctlen, taglen, aadlen;
-	unsigned char buf[4096], mac[16];
+	unsigned char buf[4096], mac[16] = { 0, }, pvsec_ct[4096];
 	const char *mkvp, *apqns[257];
 	struct zpc_aes_key *aes_key;
 	struct zpc_aes_gcm *aes_gcm;
@@ -1179,8 +1291,12 @@ TEST(aes_gcm, rederive_protected_key2)
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_KERNEL_CAPS_CHECK(type);
+
 	TESTLIB_AES_SW_CAPS_CHECK(type);
 
+	TESTLIB_APQN_CAPS_CHECK(apqns, mkvp, type, 128, flags);
+
 	rc = zpc_aes_key_alloc(&aes_key);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_gcm_alloc(&aes_gcm);
@@ -1201,8 +1317,14 @@ TEST(aes_gcm, rederive_protected_key2)
 	rc = zpc_aes_key_set_size(aes_key, keylen * 8);
 	EXPECT_EQ(rc, 0);
 
-	rc = zpc_aes_key_import_clear(aes_key, key);
-	EXPECT_EQ(rc, 0);
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		rc = zpc_aes_key_import_clear(aes_key, key);
+		EXPECT_EQ(rc, 0);
+	} else {
+		rc = testlib_set_aes_key_from_pvsecret(aes_key, keylen * 8);
+		if (rc)
+			goto ret;
+	}
 
 	/* Encrypt */
 	memcpy(buf, msg, msglen);
@@ -1223,8 +1345,12 @@ TEST(aes_gcm, rederive_protected_key2)
 	rc = zpc_aes_gcm_encrypt(aes_gcm, buf + 16, mac, taglen, NULL, 0, buf + 16, msglen - 16);
 	EXPECT_EQ(rc, 0);
 
-	EXPECT_TRUE(memcmp(buf, ct, ctlen) == 0);
-	EXPECT_TRUE(memcmp(mac, tag, taglen) == 0);
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		EXPECT_TRUE(memcmp(buf, ct, ctlen) == 0);
+		EXPECT_TRUE(memcmp(mac, tag, taglen) == 0);
+	} else {
+		memcpy(pvsec_ct, buf, ctlen);
+	}
 
 	/* Encrypt */
 	memcpy(buf, msg, msglen);
@@ -1246,11 +1372,20 @@ TEST(aes_gcm, rederive_protected_key2)
 	rc = zpc_aes_gcm_encrypt(aes_gcm, buf + 16, mac, taglen, NULL, 0, buf + 16, msglen - 16);
 	EXPECT_EQ(rc, 0);
 
-	EXPECT_TRUE(memcmp(buf, ct, ctlen) == 0);
-	EXPECT_TRUE(memcmp(mac, tag, taglen) == 0);
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		EXPECT_TRUE(memcmp(buf, ct, ctlen) == 0);
+		EXPECT_TRUE(memcmp(mac, tag, taglen) == 0);
+	} else {
+		EXPECT_TRUE(memcmp(buf, pvsec_ct, ctlen) == 0); // <- beim 2. encrypt kommt dasselbe raus
+	}
 
 	/* Decrypt */
-	memcpy(buf, ct, ctlen);
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		memcpy(buf, ct, ctlen);
+	} else {
+		memcpy(buf, pvsec_ct, ctlen);
+		memcpy(tag, mac, taglen);
+	}
 
 	memset(aes_gcm->param.protkey, 0, sizeof(aes_gcm->param.protkey));    /* force WKaVP mismatch */
 	rc = zpc_aes_gcm_set_iv(aes_gcm, iv, ivlen);
@@ -1269,7 +1404,12 @@ TEST(aes_gcm, rederive_protected_key2)
 	EXPECT_TRUE(memcmp(buf, msg, msglen) == 0);
 
 	/* Decrypt */
-	memcpy(buf, ct, ctlen);
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		memcpy(buf, ct, ctlen);
+	} else {
+		memcpy(buf, pvsec_ct, ctlen);
+		memcpy(tag, mac, taglen);
+	}
 
 	memset(aes_gcm->param.protkey, 0, sizeof(aes_gcm->param.protkey));    /* force WKaVP mismatch */
 	rc = zpc_aes_gcm_set_iv(aes_gcm, iv, ivlen);
@@ -1287,6 +1427,7 @@ TEST(aes_gcm, rederive_protected_key2)
 
 	EXPECT_TRUE(memcmp(buf, msg, msglen) == 0);
 
+ret:
 	zpc_aes_gcm_free(&aes_gcm);
 	EXPECT_EQ(aes_gcm, nullptr);
 	zpc_aes_key_free(&aes_key);
@@ -1306,8 +1447,6 @@ TEST(aes_gcm, reencipher)
 
 	TESTLIB_AES_GCM_HW_CAPS_CHECK();
 
-	TESTLIB_AES_KERNEL_CAPS_CHECK();
-
 	size_t keylen, ivlen, msglen, ctlen, taglen, aadlen;
 	unsigned char buf[4096], mac[16];
 	const char *mkvp, *apqns[257];
@@ -1328,10 +1467,17 @@ TEST(aes_gcm, reencipher)
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_KERNEL_CAPS_CHECK(type);
+
 	TESTLIB_AES_SW_CAPS_CHECK(type);
 
+	TESTLIB_APQN_CAPS_CHECK(apqns, mkvp, type, 256, flags);
+
 	TESTLIB_AES_NEW_MK_CHECK(type, mkvp, apqns);
 
+	if (type == ZPC_EC_KEY_TYPE_PVSECRET)
+		GTEST_SKIP_("Skipping reencipher test. Not applicable for UV secrets.");
+
 	u8 *key = testlib_hexstr2buf(keystr, &keylen);
 	ASSERT_NE(key, nullptr);
 	u8 *iv = testlib_hexstr2buf(ivstr, &ivlen);
@@ -1477,11 +1623,164 @@ TEST(aes_gcm, reencipher)
 	free(tag);
 }
 
+/*
+ * This test assumes that the tester manually added the clear AES key with
+ * given size to the pvsecret list file, for example:
+ *
+ * 5 AES-128-KEY:
+ *  0x8ace2a9bc6f28ae3 ...   <- secret ID
+ *  0xbe0274e3f3b36 ...    <- clear AES key
+ *  ...
+ *
+ * The test creates one pvsecret-type key and one CCA or EP11 type
+ * single AES key with the given clear key material to compare results.
+ * The specified APQN(s) decide if the single keys are CCA or EP11.
+ */
+TEST(aes_gcm, pvsecret_kat)
+{
+	struct zpc_aes_key *aes_key1, *aes_key2;
+	struct zpc_aes_gcm *ctx1, *ctx2;
+	u8 iv[12], aad[99], m[99], tag[12] = { 0, }, c[99], m_bak[99], c_bak[99];
+	const char *mkvp, *apqns[257];
+	unsigned int flags;
+	int type, type2, rc, size;
+
+	TESTLIB_ENV_AES_KEY_CHECK();
+
+	TESTLIB_AES_GCM_HW_CAPS_CHECK();
+
+	size = testlib_env_aes_key_size();
+	type = testlib_env_aes_key_type();
+	flags = testlib_env_aes_key_flags();
+	mkvp = testlib_env_aes_key_mkvp();
+	(void)testlib_env_aes_key_apqns(apqns);
+
+	if (type != ZPC_EC_KEY_TYPE_PVSECRET)
+		GTEST_SKIP_("Skipping pvsecret_kat test. Only applicable for UV secrets.");
+
+	TESTLIB_AES_KERNEL_CAPS_CHECK(type);
+
+	TESTLIB_AES_SW_CAPS_CHECK(type);
+
+	TESTLIB_APQN_CAPS_CHECK(apqns, mkvp, type, size, flags);
+
+	TESTLIB_EC_SW_CAPS_CHECK(type);
+
+	TESTLIB_EC_KERNEL_CAPS_CHECK(type, mkvp, apqns);
+
+	TESTLIB_EC_NEW_MK_CHECK(type, mkvp, apqns);
+
+	rc = zpc_aes_key_alloc(&aes_key1);
+	EXPECT_EQ(rc, 0);
+	rc = zpc_aes_key_alloc(&aes_key2);
+	EXPECT_EQ(rc, 0);
+	rc = zpc_aes_gcm_alloc(&ctx1);
+	EXPECT_EQ(rc, 0);
+	rc = zpc_aes_gcm_alloc(&ctx2);
+	EXPECT_EQ(rc, 0);
+
+	/*
+	 * key1 is created from pvsecret.
+	 */
+	rc = zpc_aes_key_set_type(aes_key1, type);
+	EXPECT_EQ(rc, 0);
+	if (mkvp != NULL) {
+		rc = zpc_aes_key_set_mkvp(aes_key1, mkvp);
+		EXPECT_EQ(rc, 0);
+	} else {
+		rc = zpc_aes_key_set_apqns(aes_key1, apqns);
+		EXPECT_EQ(rc, 0);
+	}
+
+	rc = zpc_aes_key_set_flags(aes_key1, flags);
+	EXPECT_EQ(rc, 0);
+	rc = zpc_aes_key_set_size(aes_key1, size);
+	EXPECT_EQ(rc, 0);
+
+	rc = testlib_set_aes_key_from_pvsecret(aes_key1, size);
+	if (rc)
+		goto ret;
+
+	rc = zpc_aes_gcm_set_key(ctx1, aes_key1);
+	EXPECT_EQ(rc, 0);
+
+	/*
+	 * key2 is a normal AES key and contains clear key material from list
+	 * file. We first try to create a CCA type key with the given APQN(s),
+	 * if this fails we retry with EP11.
+	 */
+	type2 = ZPC_AES_KEY_TYPE_CCA_DATA;
+	while (1) {
+		rc = zpc_aes_key_set_type(aes_key2, type2);
+		EXPECT_EQ(rc, 0);
+		if (mkvp != NULL) {
+			rc = zpc_aes_key_set_mkvp(aes_key2, mkvp);
+			EXPECT_EQ(rc, 0);
+		} else {
+			rc = zpc_aes_key_set_apqns(aes_key2, apqns);
+			EXPECT_EQ(rc, 0);
+		}
+
+		rc = zpc_aes_key_set_flags(aes_key2, flags);
+		EXPECT_EQ(rc, 0);
+
+		rc = zpc_aes_key_set_size(aes_key2, size);
+		EXPECT_EQ(rc, 0);
+
+		rc = testlib_set_aes_key_from_file(aes_key2, type2, size);
+		if (rc == ZPC_ERROR_IOCTLCLR2SECK2) {
+			type2 = ZPC_AES_KEY_TYPE_EP11;
+			continue;
+		}
+		if (rc)
+			goto ret;
+		else
+			break;
+	}
+
+	rc = zpc_aes_gcm_set_key(ctx2, aes_key2);
+	EXPECT_EQ(rc, 0);
+
+	/* Now encrypt with both keys and compare result */
+	rc = zpc_aes_gcm_set_iv(ctx1, iv, 12);
+	EXPECT_EQ(rc, 0);
+	memcpy(m_bak, m, 99);
+	rc = zpc_aes_gcm_encrypt(ctx1, c, tag, 12, aad, 99, m, 99);
+	EXPECT_EQ(rc, 0);
+	memcpy(c_bak, c, 99);
+	rc = zpc_aes_gcm_set_iv(ctx2, iv, 12);
+	EXPECT_EQ(rc, 0);
+	rc = zpc_aes_gcm_encrypt(ctx2, c, tag, 12, aad, 99, m, 99);
+	EXPECT_EQ(rc, 0);
+	EXPECT_TRUE(memcmp(c, c_bak, 99) == 0);
+
+	/* Now encrypt with key1 and decrypt with key2 */
+	rc = zpc_aes_gcm_set_iv(ctx1, iv, 12);
+	EXPECT_EQ(rc, 0);
+	rc = zpc_aes_gcm_encrypt(ctx1, c, tag, 12, aad, 99, m, 99);
+	EXPECT_EQ(rc, 0);
+	rc = zpc_aes_gcm_set_iv(ctx2, iv, 12);
+	EXPECT_EQ(rc, 0);
+	rc = zpc_aes_gcm_decrypt(ctx2, m, tag, 12, aad, 99, c, 99);
+	EXPECT_EQ(rc, 0);
+	EXPECT_TRUE(memcmp(m, m_bak, 99) == 0);
+
+ret:
+	zpc_aes_gcm_free(&ctx1);
+	EXPECT_EQ(ctx1, nullptr);
+	zpc_aes_gcm_free(&ctx2);
+	EXPECT_EQ(ctx2, nullptr);
+	zpc_aes_key_free(&aes_key1);
+	EXPECT_EQ(aes_key1, nullptr);
+	zpc_aes_key_free(&aes_key2);
+	EXPECT_EQ(aes_key2, nullptr);
+}
+
 static void
 __task(struct zpc_aes_key *aes_key)
 {
 	struct zpc_aes_gcm *aes_gcm;
-	unsigned char buf[4096], mac[16];
+	unsigned char buf[4096], mac[16] = { 0,};
 	size_t ivlen, msglen, ctlen, taglen, aadlen;
 	int rc, i;
 
@@ -1524,11 +1823,17 @@ __task(struct zpc_aes_key *aes_key)
 		rc = zpc_aes_gcm_encrypt(aes_gcm, buf + 16, mac, taglen, NULL, 0, buf + 16, msglen - 16);
 		EXPECT_EQ(rc, 0);
 
-		EXPECT_TRUE(memcmp(buf, ct, ctlen) == 0);
-		EXPECT_TRUE(memcmp(mac, tag, taglen) == 0);
+		if (aes_key->type != ZPC_AES_KEY_TYPE_PVSECRET) {
+			EXPECT_TRUE(memcmp(buf, ct, ctlen) == 0);
+			EXPECT_TRUE(memcmp(mac, tag, taglen) == 0);
+		}
 
 		/* Decrypt */
-		memcpy(buf, ct, ctlen);
+		if (aes_key->type != ZPC_AES_KEY_TYPE_PVSECRET) {
+			memcpy(buf, ct, ctlen);
+		} else {
+			memcpy(tag, mac, taglen);
+		}
 
 		rc = zpc_aes_gcm_set_iv(aes_gcm, iv, ivlen);
 		EXPECT_EQ(rc, 0);
@@ -1560,8 +1865,6 @@ TEST(aes_gcm, threads)
 
 	TESTLIB_AES_GCM_HW_CAPS_CHECK();
 
-	TESTLIB_AES_KERNEL_CAPS_CHECK();
-
 	size_t keylen;
 	const char *mkvp, *apqns[257];
 	struct zpc_aes_key *aes_key;
@@ -1576,8 +1879,12 @@ TEST(aes_gcm, threads)
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_KERNEL_CAPS_CHECK(type);
+
 	TESTLIB_AES_SW_CAPS_CHECK(type);
 
+	TESTLIB_APQN_CAPS_CHECK(apqns, mkvp, type, 128, flags);
+
 	TESTLIB_AES_NEW_MK_CHECK(type, mkvp, apqns);
 
 	u8 *key = testlib_hexstr2buf(keystr, &keylen);
@@ -1600,17 +1907,25 @@ TEST(aes_gcm, threads)
 	rc = zpc_aes_key_set_size(aes_key, keylen * 8);
 	EXPECT_EQ(rc, 0);
 
-	rc = zpc_aes_key_import_clear(aes_key, key);
-	EXPECT_EQ(rc, 0);
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		rc = zpc_aes_key_import_clear(aes_key, key);
+		EXPECT_EQ(rc, 0);
+	} else {
+		rc = testlib_set_aes_key_from_pvsecret(aes_key, keylen * 8);
+		if (rc)
+			goto ret;
+	}
 
 	for (i = 0; i < 500; i++) {
 		t[i] = new std::thread(__task, aes_key);
 	}
 
 	/* Do something with key object while threads are working with it. */
-	rc = zpc_aes_key_reencipher(aes_key, ZPC_AES_KEY_REENCIPHER_CURRENT_TO_NEW);
-	EXPECT_EQ(rc, 0);
-	memset(&aes_key->cur, 0, sizeof(aes_key->cur));     /* destroy current secure key */
+	if (type != ZPC_AES_KEY_TYPE_PVSECRET) {
+		rc = zpc_aes_key_reencipher(aes_key, ZPC_AES_KEY_REENCIPHER_CURRENT_TO_NEW);
+		EXPECT_EQ(rc, 0);
+		memset(&aes_key->cur, 0, sizeof(aes_key->cur));     /* destroy current secure key */
+	}
  
 	for (i = 0; i < 500; i++) {
 		memset(&aes_key->prot, 0, sizeof(aes_key->prot));    /* destroy cached protected key */
@@ -1622,6 +1937,7 @@ TEST(aes_gcm, threads)
 		delete t[i];
 	}
 
+ret:
 	zpc_aes_key_free(&aes_key);
 	EXPECT_EQ(aes_key, nullptr);
 
